require 'rubygame'

class Circle
  def initialize(x, y, deltaX, deltaY, radius, world)
    @x = x
    @y = y
    @deltaX = deltaX
    @deltaY = deltaY
    @radius = radius
    
    @delta = 1
    
    @world = world
    @color = [rand(180)+50, rand(180)+50, rand(180)+50, rand(175)+80]
    @weight = @radius*@radius*Math::PI
  end
  
  def move
    updateXY
    updateDeltaXY
    checkDeleteSelf
  end
  
  def updateXY
    delta = @world.delta.to_f
    @x += @deltaX*delta
    @y += @deltaY*delta
  end
  
  def updateDeltaXY
    @deltaY += (@world.gravity*(@weight/100.0))
    
    if @x < @radius
      @deltaX = (-@deltaX*0.95)
      @x = @radius+1
    elsif @x > @world.screen.width-@radius
      @deltaX = (-@deltaX*0.95)
      @x = @world.screen.width-@radius-1
    end
    
    if @y < @radius
      @deltaY = (-@deltaY*0.95)
      @y = @radius+1
    elsif @y > @world.screen.height-@radius
      @deltaY = (-@deltaY*0.95)
      @y = @world.screen.height-@radius-1
    end
  end
  
  def checkDeleteSelf
    if @deltaY.abs < 0.01 and @y > @world.screen.height-1
      @world.deleteSquare(self)
    end
  end
  
  def draw
    Rubygame::Draw.filled_circle(
      @world.screen,
      [@x, @y],
      @radius,
      @color
    )
  end
end

class DeltaTime
  PERFECT_TIME = 1000.0/60.0
  
  def initialize
    @oldTime = Rubygame::Time.get_ticks
    @delta = PERFECT_TIME
  end
  
  def update
    newTime = Rubygame::Time.get_ticks
    @delta = (newTime-@oldTime) / PERFECT_TIME
    #while Rubygame::Time.get_ticks-@oldTime < PERFECT_TIME
    #end
    @oldTime = newTime
  end
  
  def to_f
    @delta.to_f
  end
  
  def to_s
    @delta.to_s
  end
end

class World
  DOWN = 0
  UP = 1
  MOMENTUM_SIZE = 6
  SPEED = 2
  MOUSE_POSITION_COUNT = 5
  MOUSE_DELAY = 5
  
  GRAVITY = 0.00098
  
  def initialize
    Rubygame.init
    @delta = DeltaTime.new
    @screen = Rubygame::Screen.set_mode( [800, 600], 32, [Rubygame::DOUBLEBUF] )
    @circles = []
    
    @mousePosition = []
    @mousePositionCounter = 0
    @mouseDelay = 0
    
    @mouseLeft = UP
    @mouseRight = UP
    
  end
  
  def mainloop
    while true
      @delta.update
      setCaption
      @screen.fill( [0, 0, 0] )
        
        controls
        
        if @mouseLeft == DOWN and !@mousePosition[1].nil?
          newCircle(@mousePosition[0])
        end
        
        @circles.each do |circle|
          circle.draw
          circle.move
        end
        
      @screen.flip
    end
  end
  
  def delta
    @delta
  end
  
  def setCaption
    @screen.set_caption("Ruby Circles: #{@circles.size}")
  end
  
  def newCircle(pos)
    if @mouseDelay == MOUSE_DELAY
      size = ((rand+0.2)*rand(4) + 2)**2
      deltaXY = getXY
      
      @circles << Circle.new(
        pos[0], pos[1], #X/Y location
        deltaXY[0], deltaXY[1], #Delta X/Y
        size, #Width/Height
        self #Refference to the world
      )
      
      @mouseDelay = 0
    else
      @mouseDelay += 1
    end
  end
  
  def mouseLeftUp
    @mouseLeft = UP
  end
  
  def mouseLeftDown(pos)
    @mouseLeft = DOWN
  end
  
  def mouseRightUp
    @mouseLeft = UP
  end
  
  def mouseRightDown
    unless @mouseRight == DOWN
      @mouseRight = DOWN
    end
  end
  
  def mouseMomentum(pos)
    @mousePositionCounter += 1
    if @mousePositionCounter == MOUSE_POSITION_COUNT and !@mousePositionCounter[1].nil?
      @mousePosition[0], @mousePosition[1] = pos, @mousePosition[0]
      @mousePositionCounter = 0
    end
  end
  
  def getXY
    angle = Math.atan2( @mousePosition[0][1]-@mousePosition[1][1], @mousePosition[0][0]-@mousePosition[1][0] )
    [SPEED*Math.cos(angle), SPEED*Math.sin(angle)]
  end
  
  def controls
    Rubygame::Queue.instance.get.each do |event|
      case (event)
      # MOUSE
      when Rubygame::MouseDownEvent
        mouseMomentum(event.pos)
        
        case event.button
        when Rubygame::MOUSE_LEFT
          mouseLeftDown(event.pos)
        when Rubygame::MOUSE_RIGHT
          mouseRightDown
        end
          
      when Rubygame::MouseUpEvent
        mouseMomentum(event.pos)
        
        case event.button
        when Rubygame::MOUSE_LEFT
          mouseLeftUp
        when Rubygame::MOUSE_RIGHT
          mouseRightUp
        end
        
      when Rubygame::MouseMotionEvent
        mouseMomentum(event.pos)
        
      # KEYBOARD
      when Rubygame::KeyDownEvent
        case event.key
        when Rubygame::K_ESCAPE
          exit
        end
      end
    end
    
    def deleteCircle(circle)
      @circles.delete(circle)
    end
    
    def gravity
      GRAVITY
    end
    
    def screen
      @screen
    end
  end
end

World.new.mainloop